Module: Elf
- Defined in:
- lib/elf/gnu.rb,
lib/elf.rb,
lib/elf/file.rb,
lib/elf/sunw.rb,
lib/elf/tools.rb,
lib/elf/value.rb,
lib/elf/symbol.rb,
lib/elf/dynamic.rb,
lib/elf/section.rb,
lib/elf/section.rb,
lib/elf/stringtable.rb,
lib/elf/symboltable.rb,
lib/elf/utils/loader.rb,
lib/elf/symbol/demangler_gcc3.rb
Overview
This file provides some utilities to deal with the runtime lodader functions. In particular it provides access to the same kind of library search as the loader provides.
Defined Under Namespace
Modules: GNU, SunW, Utilities Classes: Class, DataEncoding, Dynamic, File, Indexes, Machine, OsAbi, Section, StringTable, Symbol, SymbolTable, Tool, Value
Constant Summary collapse
- VERSION =
"1.0.6"- MagicString =
"\177ELF"