Class: Elf::Utilities::FilePool
- Inherits:
-
Object
- Object
- Elf::Utilities::FilePool
- Defined in:
- lib/elf/utils/pool.rb
Overview
Pool for ELF files.
This pool is useful for tools that recurse over a tree of dependencies to avoid creating multiple instances of Elf::File accessing the same file.